import functools
import logging
import os

from azure.identity import EnvironmentCredential
from azure.mgmt.containerservice import ContainerServiceClient

from qhub.provider.cloud.commons import filter_by_highest_supported_k8s_version

logger = logging.getLogger("azure")
logger.setLevel(logging.ERROR)


@functools.lru_cache()
def initiate_container_service_client():
    subscription_id = os.environ.get("ARM_SUBSCRIPTION_ID", None)

    # Python SDK needs different env var names to Terraform SDK
    for envname in ("TENANT_ID", "CLIENT_SECRET", "CLIENT_ID"):
        azure_name = f"AZURE_{envname}"
        if azure_name not in os.environ:
            os.environ[azure_name] = os.environ[f"ARM_{envname}"]

    return ContainerServiceClient(
        credential=EnvironmentCredential(), subscription_id=subscription_id
    )


@functools.lru_cache()
def kubernetes_versions(region="Central US"):
    """Return list of available kubernetes supported by cloud provider. Sorted from oldest to latest."""

    client = initiate_container_service_client()
    azure_location = region.replace(" ", "").lower()

    k8s_versions_list = client.container_services.list_orchestrators(
        azure_location, resource_type="managedClusters"
    ).as_dict()
    supported_kubernetes_versions = []

    for key in k8s_versions_list["orchestrators"]:
        if key["orchestrator_type"] == "Kubernetes":
            supported_kubernetes_versions.append(key["orchestrator_version"])

    supported_kubernetes_versions = sorted(supported_kubernetes_versions)
    return filter_by_highest_supported_k8s_version(supported_kubernetes_versions)



import ast
import logging

from qhub.cost import infracost_report

logger = logging.getLogger(__name__)


def create_cost_subcommand(subparser):
    subparser = subparser.add_parser("cost-estimate")
    subparser.add_argument(
        "-p",
        "--path",
        help="Pass the path of your stages directory generated after rendering QHub configurations before deployment",
        required=False,
    )
    subparser.add_argument(
        "-d",
        "--dashboard",
        default="True",
        help="Enable the cost dashboard",
        required=False,
    )
    subparser.add_argument(
        "-f",
        "--file",
        help="Specify the path of the file to store the cost report",
        required=False,
    )
    subparser.add_argument(
        "-c",
        "--currency",
        help="Specify the currency code to use in the cost report",
        required=False,
        default="USD",
    )
    subparser.add_argument(
        "-cc",
        "--compare",
        help="Compare the cost report to a previously generated report",
        required=False,
    )
    subparser.set_defaults(func=handle_cost_report)


def handle_cost_report(args):
    infracost_report(
        args.path,
        ast.literal_eval(args.dashboard),
        args.file,
        args.currency,
        args.compare,
    )
